@@ -15,11 +15,11 @@ except ImportError:
 
 import openmdao
 from openmdao.core.problem import Problem
-from openmdao.core.notebook_mode import notebook, colab
 from openmdao.utils.units import convert_units
 from openmdao.utils.mpi import MPI
 from openmdao.utils.webview import webview
 from openmdao.utils.general_utils import printoptions
+from openmdao.utils.notebook_utils import notebook, colab
 
 
 def _val2str(val):
@@ -41,7 +41,7 @@ def view_connections(root, outfile='connections.html', show_browser=True,
 
     Parameters
     ----------
-    root : system or Problem
+    root : System or Problem
         The root for the desired tree.
 
     outfile : str, optional
